@@ -24,16 +24,14 @@
 
 #include "farms.h"
 
-#define NFTLB_POSTROUTING_MARK		0x80000000
-
-int get_farm_mark(struct farm *f);
-int get_bck_mark(struct backend *b);
+#define NFTLB_MASQUERADE_MARK_DEFAULT		0x80000000
 
 int nft_reset(void);
 int nft_check_tables(void);
-int nft_rulerize(struct farm *f);
+int nft_rulerize_farms(struct farm *f);
+int nft_rulerize_address(struct address *a);
 int nft_rulerize_policies(struct policy *p);
-int nft_get_rules_buffer(const char **buf, int key, struct farm *f, struct policy *p);
+int nft_get_rules_buffer(const char **buf, int key, struct nftst *n);
 void nft_del_rules_buffer(const char *buf);
 
 #endif /* _NFT_H_ */
