@@ -22,18 +22,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 
 #include "elements.h"
 #include "policies.h"
 #include "objects.h"
+#include "tools.h"
 #include "nft.h"
 
-static struct element * element_create(struct policy *p, char *data, char *time)
+static struct element * element_create(struct policy *p, char *data, char *time, char *counter_pkts, char *counter_bytes)
 {
 	struct element *e = (struct element *)malloc(sizeof(struct element));
 	if (!e) {
-		syslog(LOG_ERR, "element memory allocation error");
+		tools_printlog(LOG_ERR, "element memory allocation error");
 		return NULL;
 	}
 
@@ -44,6 +44,8 @@ static struct element * element_create(struct policy *p, char *data, char *time)
 	e->time = DEFAULT_ELEMENT_TIME;
 	if (time && strcmp(time, "") != 0)
 		obj_set_attribute_string(time, &e->time);
+	obj_set_attribute_string(counter_pkts, &e->counter_pkts);
+	obj_set_attribute_string(counter_bytes, &e->counter_bytes);
 
 	list_add_tail(&e->list, &p->elements);
 	p->total_elem++;
@@ -56,8 +58,12 @@ static int element_delete_node(struct element *e)
 	list_del(&e->list);
 	if (e->data)
 		free(e->data);
-	if (e->time && strcmp(e->time, "") != 0)
+	if (e->time)
 		free(e->time);
+	if (e->counter_pkts)
+		free(e->counter_pkts);
+	if (e->counter_bytes)
+		free(e->counter_bytes);
 
 	free(e);
 
@@ -66,6 +72,9 @@ static int element_delete_node(struct element *e)
 
 static int element_delete(struct element *e)
 {
+	if (!e)
+		return 0;
+
 	struct policy *p = e->policy;
 
 	p->total_elem--;
@@ -79,11 +88,11 @@ static int element_delete(struct element *e)
 static int nft_parse_elements(struct policy *p, const char *buf)
 {
 	char *ini_ptr = NULL;
-	char *fin_ptr = NULL;
-	char element1[2550] = {0};
-	char element2[2550] = {0};
-	char element3[2550] = {0};
-	int next = 0;
+	char *fin_ptr = NULL, *fin1_ptr = NULL;
+	char elem_addr[100] = {0};
+	char elem_time[100] = {0};
+	char elem_pkts[100] = {0};
+	char elem_bytes[100] = {0};
 
 	ini_ptr = strstr(buf, "elements = { ");
 	if (ini_ptr == NULL)
@@ -91,38 +100,40 @@ static int nft_parse_elements(struct policy *p, const char *buf)
 
 	ini_ptr += 13;
 new_element:
-	next = 0;
 
-	if (p->timeout) {
-		if ((fin_ptr = strstr(ini_ptr, " expires ")) != NULL) {
-			snprintf(element1, fin_ptr - ini_ptr + 1, "%s", ini_ptr);
-			fin_ptr += 9;
-			ini_ptr = fin_ptr;
-		} else
-			return 0;
+	fin_ptr = strstr(ini_ptr, ",");
+	if ((fin1_ptr = strstr(ini_ptr, " ")) != NULL) {
+		if (!fin_ptr || (fin1_ptr < fin_ptr))
+			fin_ptr = fin1_ptr;
 	}
 
-	if ((fin_ptr = strstr(ini_ptr + strlen(element2), ",")) != NULL) {
-		next = 1;
-	} else {
-		if ((fin_ptr = strstr(ini_ptr, " ")) == NULL)
-			return 0;
-	}
-
-	snprintf(element3, fin_ptr - ini_ptr + 1, "%s", ini_ptr);
+	tools_snprintf(elem_addr, fin_ptr - ini_ptr, ini_ptr);
 	fin_ptr += 1;
 	ini_ptr = fin_ptr;
 
-	if (p->timeout)
-		element_create(p, element3, element2);
-	else
-		element_create(p, element3, NULL);
-
-	while (*fin_ptr == '\n' || *fin_ptr == '\t' || *fin_ptr == ' ') {
-		fin_ptr++;
+	if ((fin_ptr = strstr(ini_ptr, " bytes ")) != NULL) {
+		ini_ptr += 16;
+		tools_snprintf(elem_pkts, fin_ptr - ini_ptr, ini_ptr);
+		ini_ptr += 8;
+		if ((fin_ptr = strstr(ini_ptr, ",")) != NULL || (fin_ptr = strstr(ini_ptr, " ")) != NULL) {
+			tools_snprintf(elem_bytes, fin_ptr - ini_ptr, ini_ptr);
+			ini_ptr = ++fin_ptr;
+		}
+	} else {
+		tools_snprintf(elem_pkts, 3, DEFAULT_COUNTER);
+		tools_snprintf(elem_bytes, 3, DEFAULT_COUNTER);
+		fin_ptr = ini_ptr;
 	}
 
-	if (next && (*fin_ptr != '}' || *fin_ptr != '\0')) {
+	if (p->timeout)
+		element_create(p, elem_addr, elem_time, elem_pkts, elem_bytes);
+	else
+		element_create(p, elem_addr, NULL, elem_pkts, elem_bytes);
+
+	while (*fin_ptr == '\n' || *fin_ptr == '\t' || *fin_ptr == ' ')
+		fin_ptr++;
+
+	if (*fin_ptr != '}' && *fin_ptr != '\0') {
 		ini_ptr = fin_ptr;
 		goto new_element;
 	}
@@ -135,11 +146,13 @@ void element_s_print(struct policy *p)
 	struct element *e;
 
 	list_for_each_entry(e, &p->elements, list) {
-		syslog(LOG_DEBUG,"    [element] ");
-		syslog(LOG_DEBUG,"       [data] %s", e->data);
+		tools_printlog(LOG_DEBUG,"    [element] ");
+		tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_DATA, e->data);
 		if (p->timeout && e->time && strcmp(e->time, "") != 0)
-			syslog(LOG_DEBUG,"       [time] %s", e->time);
-		syslog(LOG_DEBUG,"       *[action] %d", e->action);
+			tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_TIME, e->time);
+		tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_COUNTER_PACKETS, e->counter_pkts);
+		tools_printlog(LOG_DEBUG,"       [%s] %s", CONFIG_KEY_COUNTER_BYTES, e->counter_bytes);
+		tools_printlog(LOG_DEBUG,"       *[action] %d", e->action);
 	}
 }
 
@@ -157,7 +170,7 @@ struct element * element_lookup_by_name(struct policy *p, const char *data)
 
 int element_set_action(struct element *e, int action)
 {
-	syslog(LOG_DEBUG, "%s():%d: element %s action is %d - new action %d", __FUNCTION__, __LINE__, e->data, e->action, action);
+	tools_printlog(LOG_DEBUG, "%s():%d: element %s action is %d - new action %d", __FUNCTION__, __LINE__, e->data, e->action, action);
 
 	if (action == ACTION_DELETE) {
 		element_delete(e);
@@ -194,12 +207,12 @@ int element_s_delete(struct policy *p)
 	list_for_each_entry_safe(e, next, &p->elements, list)
 		element_delete(e);
 
-	//~ p->total_elem = 0;
+	p->total_elem = 0;
 
 	return 0;
 }
 
-int element_set_attribute(struct config_pair *c)
+int element_set_attribute(struct config_pair *c, int apply_action)
 {
 	struct policy *p = obj_get_current_policy();
 	struct element *e = obj_get_current_element();
@@ -209,15 +222,14 @@ int element_set_attribute(struct config_pair *c)
 
 	switch (c->key) {
 	case KEY_DATA:
-		e = element_lookup_by_name(p, c->str_value);
-		if (!e) {
-			e = element_create(p, c->str_value, NULL);
-			if (!e)
-				return -1;
-		}
+		e = element_create(p, c->str_value, NULL, DEFAULT_COUNTER, DEFAULT_COUNTER);
+		if (!e)
+			return -1;
 		obj_set_current_element(e);
 		break;
 	case KEY_TIME:
+		if (e->time)
+			free(e->time);
 		obj_set_attribute_string(c->str_value, &e->time);
 		break;
 	case KEY_ACTION:
@@ -230,7 +242,7 @@ int element_set_attribute(struct config_pair *c)
 	return 0;
 }
 
-int element_pos_actionable(struct config_pair *c)
+int element_pos_actionable(struct config_pair *c, int apply_action)
 {
 	struct policy *p = obj_get_current_policy();
 	struct element *e = obj_get_current_element();
@@ -238,9 +250,19 @@ int element_pos_actionable(struct config_pair *c)
 	if (!p || !e)
 		return -1;
 
-	syslog(LOG_DEBUG, "%s():%d: pos actionable element %s of policy %s with param %d", __FUNCTION__, __LINE__, e->data, p->name, c->key);
+	tools_printlog(LOG_DEBUG, "%s():%d: pos actionable element %s of policy %s with param %d", __FUNCTION__, __LINE__, e->data, p->name, c->key);
 
-	policy_set_action(p, ACTION_RELOAD);
+	switch (c->key) {
+	case KEY_DATA:
+		if (apply_action != ACTION_START) {
+			element_set_action(e, apply_action);
+			policy_set_action(p, ACTION_RELOAD);
+		}
+		break;
+	default:
+		policy_set_action(p, ACTION_RELOAD);
+		break;
+	}
 
 	return 0;
 }
@@ -248,12 +270,15 @@ int element_pos_actionable(struct config_pair *c)
 int element_get_list(struct policy *p)
 {
 	const char *buf;
-	syslog(LOG_DEBUG, "%s():%d: policy %s", __FUNCTION__, __LINE__, p->name);
+	struct nftst *n = nftst_create_from_policy(p);
 
-	nft_get_rules_buffer(&buf, KEY_POLICIES, NULL, p);
+	tools_printlog(LOG_DEBUG, "%s():%d: policy %s", __FUNCTION__, __LINE__, p->name);
+
+	nft_get_rules_buffer(&buf, KEY_POLICIES, n);
 	p->total_elem = 0;
 	nft_parse_elements(p, buf);
 	nft_del_rules_buffer(buf);
 	element_s_print(p);
+	nftst_delete(n);
 	return 0;
 }
