@@ -24,7 +24,6 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <ev.h>
-#include <syslog.h>
 #include <time.h>
 #include <unistd.h>
 #include <netdb.h>
@@ -36,8 +35,9 @@
 #include "nft.h"
 #include "events.h"
 #include "sbuffer.h"
+#include "tools.h"
 
-#define SRV_MAX_BUF			40960
+#define SRV_MAX_BUF				1024
 #define SRV_MAX_HEADER			300
 #define SRV_MAX_IDENT			200
 #define SRV_KEY_LENGTH			16
@@ -48,6 +48,7 @@
 #define STR_POST_ACTION			"POST"
 #define STR_PUT_ACTION			"PUT"
 #define STR_DELETE_ACTION		"DELETE"
+#define STR_PATCH_ACTION		"PATCH"
 
 #define HTTP_PROTO			"HTTP/1.1 "
 #define HTTP_LINE_END			"\r\n"
@@ -61,6 +62,7 @@ enum ws_methods {
 	WS_POST_ACTION,
 	WS_PUT_ACTION,
 	WS_DELETE_ACTION,
+	WS_PATCH_ACTION,
 };
 
 enum ws_responses {
@@ -88,7 +90,6 @@ static const char *ws_str_responses[] = {
 };
 
 struct nftlb_server {
-	int			clients;
 	char			*key;
 	int			family;
 	char			*host;
@@ -102,6 +103,23 @@ static struct nftlb_server nftserver = {
 	.port	= NULL,
 };
 
+static int parse_to_http_status(int code)
+{
+	switch (code) {
+	case PARSER_OK:
+	case PARSER_IDEM_VALUE:
+		return WS_HTTP_200;
+	case PARSER_STRUCT_FAILED:
+	case PARSER_VALID_FAILED:
+		return WS_HTTP_400;
+	case PARSER_OBJ_UNKNOWN:
+		return WS_HTTP_404;
+	case PARSER_FAILED:
+	default:
+		return WS_HTTP_500;
+	}
+}
+
 static int auth_key(const char *recvkey)
 {
 	return (strcmp(nftserver.key, recvkey) == 0);
@@ -134,29 +152,30 @@ static int get_request(int fd, struct sbuffer *buf, struct nftlb_http_state *sta
 
 	sscanf(get_buf_data(buf), "%199[^ ] %199[^ ] ", method, state->uri);
 
-	syslog(LOG_NOTICE, "%s():%d: request: %s %s", __FUNCTION__, __LINE__, method, state->uri);
+	tools_printlog(LOG_NOTICE, "%s():%d: request: %s %s", __FUNCTION__, __LINE__, method, state->uri);
 
-	if (strncmp(method, STR_GET_ACTION, 3) == 0) {
+	if (strncmp(method, STR_GET_ACTION, 4) == 0) {
 		state->method = WS_GET_ACTION;
-	} else if (strncmp(method, STR_POST_ACTION, 4) == 0) {
+	} else if (strncmp(method, STR_POST_ACTION, 5) == 0) {
 		state->method = WS_POST_ACTION;
-	} else if (strncmp(method, STR_PUT_ACTION, 5) == 0) {
+	} else if (strncmp(method, STR_PUT_ACTION, 4) == 0) {
 		state->method = WS_PUT_ACTION;
-	} else if (strncmp(method, STR_DELETE_ACTION, 6) == 0) {
+	} else if (strncmp(method, STR_DELETE_ACTION, 7) == 0) {
 		state->method = WS_DELETE_ACTION;
+	} else if (strncmp(method, STR_PATCH_ACTION, 6) == 0) {
+		state->method = WS_PATCH_ACTION;
 	} else {
-		state->status_code = WS_HTTP_500;
+		state->status_code = parse_to_http_status(PARSER_STRUCT_FAILED);
 		return -1;
 	}
 
-	if (state->method != WS_POST_ACTION &&
-	    state->method != WS_PUT_ACTION)
+	if (state->method == WS_GET_ACTION)
 		return 0;
 
 	state->body = strstr(get_buf_data(buf), "\r\n\r\n");
 	if (!state->body) {
-		syslog(LOG_ERR, "Not found body section in the request");
-		state->status_code = WS_HTTP_400;
+		tools_printlog(LOG_ERR, "Not found body section in the request");
+		state->status_code = parse_to_http_status(PARSER_STRUCT_FAILED);
 		return -1;
 	}
 	state->body += 4;
@@ -176,7 +195,7 @@ static int get_request(int fd, struct sbuffer *buf, struct nftlb_http_state *sta
 			goto receive;
 
 		if (resize_buf(buf, times)) {
-			syslog(LOG_ERR, "Error resizing the buffer %d times from a size of %d!", times, get_buf_size(buf));
+			tools_printlog(LOG_ERR, "Error resizing the buffer %d times from a size of %d!", times, get_buf_size(buf));
 			state->status_code = WS_HTTP_500;
 			return -1;
 		}
@@ -209,56 +228,50 @@ static int init_http_state(struct nftlb_http_state *state)
 {
 	state->body_response = malloc(SRV_MAX_BUF);
 	if (!state->body_response) {
-		state->status_code = WS_HTTP_500;
+		state->status_code = parse_to_http_status(PARSER_STRUCT_FAILED);
 		return -1;
 	}
 
 	return 0;
 }
 
+static int fin_http_state(struct nftlb_http_state *state)
+{
+	if (state->body_response)
+		free(state->body_response);
+	return 0;
+}
+
 static int send_get_response(struct nftlb_http_state *state)
 {
 	char firstlevel[SRV_MAX_IDENT] = {0};
 	char secondlevel[SRV_MAX_IDENT] = {0};
 	char thirdlevel[SRV_MAX_IDENT] = {0};
 	char fourthlevel[SRV_MAX_IDENT] = {0};
+	int ret = PARSER_STRUCT_FAILED;
 
 	sscanf(state->uri, "/%199[^/]/%199[^/]/%199[^/]/%199[^\n]",
 	       firstlevel, secondlevel, thirdlevel, fourthlevel);
 
-	if (init_http_state(state))
-		return -1;
-
 	if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0) {
 
-		if (strcmp(thirdlevel, CONFIG_KEY_SESSIONS) == 0 &&
-			config_print_farm_sessions(&state->body_response, secondlevel) == 0) {
-				state->status_code = WS_HTTP_200;
-				return 0;
-		}
+		if (strcmp(thirdlevel, CONFIG_KEY_SESSIONS) == 0)
+			ret = config_print_farm_sessions(&state->body_response, secondlevel);
+		else if (strcmp(thirdlevel, "") == 0)
+			ret = config_print_farms(&state->body_response, secondlevel);
 
-		if (strcmp(thirdlevel, "") == 0 &&
-			config_print_farms(&state->body_response, secondlevel) == 0) {
-				state->status_code = WS_HTTP_200;
-				return 0;
-		}
+	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0)
+		ret = config_print_policies(&state->body_response, secondlevel);
 
+	else if (strcmp(firstlevel, CONFIG_KEY_ADDRESSES) == 0)
+		ret = config_print_addresses(&state->body_response, secondlevel);
+
+	state->status_code = parse_to_http_status(ret);
+	if (ret) {
 		config_print_response(&state->body_response, "%s%s", "invalid request",
-								config_get_output());
+							  config_get_output());
 		config_delete_output();
-		state->status_code = WS_HTTP_404;
-		return 0;
-
-	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
-			   config_print_policies(&state->body_response, secondlevel) == 0) {
-		state->status_code = WS_HTTP_200;
-		return 0;
 	}
-
-	config_print_response(&state->body_response, "%s%s", "invalid request",
-							config_get_output());
-	config_delete_output();
-	state->status_code = WS_HTTP_404;
 	return 0;
 }
 
@@ -268,196 +281,273 @@ static int send_delete_response(struct nftlb_http_state *state)
 	char secondlevel[SRV_MAX_IDENT] = {0};
 	char thirdlevel[SRV_MAX_IDENT] = {0};
 	char fourthlevel[SRV_MAX_IDENT] = {0};
-	int ret;
+	char fifthlevel[SRV_MAX_IDENT] = {0};
+	char message[SRV_MAX_IDENT] = {0};
+	int ret = PARSER_OK;
 
-	sscanf(state->uri, "/%199[^/]/%199[^/]/%199[^/]/%199[^\n]",
-	       firstlevel, secondlevel, thirdlevel, fourthlevel);
-
-	if (init_http_state(state))
-		return -1;
+	sscanf(state->uri, "/%199[^/]/%199[^/]/%199[^/]/%199[^/]/%199[^\n]",
+	       firstlevel, secondlevel, thirdlevel, fourthlevel, fifthlevel);
 
 	if (strcmp(firstlevel, CONFIG_KEY_FARMS) != 0 &&
-		strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0) {
-		config_print_response(&state->body_response, "%s%s", "invalid request",
-								config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_404;
-		return 0;
+		strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0 &&
+		strcmp(firstlevel, CONFIG_KEY_ADDRESSES) != 0) {
+		snprintf(message, SRV_MAX_IDENT, "%s", "invalid request");
+		ret = PARSER_OBJ_UNKNOWN;
+		goto delete_end;
 	}
 
-	if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
-		strcmp(thirdlevel, CONFIG_KEY_BCKS) == 0) {
-		ret = config_set_backend_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error deleting backend",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
+	snprintf(message, SRV_MAX_IDENT, "%s", "success");
+	if (strlen(state->body) == 0) {
 
-	} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
-		strcmp(thirdlevel, CONFIG_KEY_SESSIONS) == 0) {
-		ret = config_set_session_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error deleting session",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
+		if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+			strcmp(thirdlevel, CONFIG_KEY_BCKS) == 0 &&
+			strcmp(fifthlevel, CONFIG_KEY_SESSIONS) == 0) {
+			ret = config_set_session_backend_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
+			if (ret)
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting sessions from backend");
 
-	} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
-			   strcmp(thirdlevel, CONFIG_KEY_POLICIES) == 0) {
-		ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error reloading farm",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
-		ret = config_set_fpolicy_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
-		if (ret != 0) {
-			config_print_response(&state->body_response, "%s%s", "error stopping farm policy",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
-		obj_rulerize(OBJ_START);
+		} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+			strcmp(thirdlevel, CONFIG_KEY_BCKS) == 0) {
+			ret = config_set_backend_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
+			if (ret)
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting backend");
 
-	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
-			   strcmp(thirdlevel, CONFIG_KEY_ELEMENTS) == 0) {
-		ret = config_get_elements(secondlevel);
-		if (ret != 0) {
-			config_print_response(&state->body_response, "%s%s", "could not get the policy elements",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
-		ret = config_set_element_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error deleting policy element",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
+		} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+			strcmp(thirdlevel, CONFIG_KEY_SESSIONS) == 0) {
+			ret = config_set_session_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret)
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting session");
+
+		} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+			strcmp(thirdlevel, CONFIG_KEY_ADDRESSES) == 0) {
+			config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+			ret = config_set_farmaddress_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting farm address");
+				goto delete_end;
+			}
+			obj_rulerize(OBJ_START);
+			config_set_farmaddress_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
+
+		} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+				   strcmp(thirdlevel, CONFIG_KEY_POLICIES) == 0) {
+			config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+			ret = config_set_fpolicy_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting farm policy");
+				goto delete_end;
+			}
+			obj_rulerize(OBJ_START);
+			config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+			config_set_fpolicy_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_DELETE);
+
+		} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
+				   strcmp(thirdlevel, CONFIG_KEY_ELEMENTS) == 0) {
+			ret = config_get_elements(secondlevel);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "could not get the policy elements");
+				goto delete_end;
+			}
+
+			// subnet support
+			if (strcmp(fifthlevel,"") != 0 &&
+				(strlen(fourthlevel) + strlen(fifthlevel) + 1 < SRV_MAX_IDENT)) {
+				strcat(fourthlevel, "/");
+				strcat(fourthlevel, fifthlevel);
+			}
+
+			ret = config_set_element_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting policy element");
+				config_delete_elements(secondlevel);
+				goto delete_end;
+			}
+			if (strcmp(fourthlevel, "") != 0) {
+				config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
+				ret = config_set_element_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_STOP);
+				if (ret) {
+					snprintf(message, SRV_MAX_IDENT, "%s", "error deleting policy element");
+					config_delete_elements(secondlevel);
+					goto delete_end;
+				}
+			} else {
+				config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_FLUSH);
+				config_set_element_action(secondlevel, fourthlevel, CONFIG_VALUE_ACTION_NONE);
+			}
+			obj_rulerize(OBJ_START);
 			config_delete_elements(secondlevel);
-			goto delete_end;
-		}
-		config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_RELOAD);
-		obj_rulerize(OBJ_START);
-		config_delete_elements(secondlevel);
 
-	} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
-			   strcmp(thirdlevel, "") == 0) {
-		ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error deleting farm",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
-		obj_rulerize(OBJ_START);
-		config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
+		} else if (strcmp(firstlevel, CONFIG_KEY_FARMS) == 0 &&
+				   strcmp(thirdlevel, "") == 0) {
+			ret = config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting farm");
+				goto delete_end;
+			}
+			obj_rulerize(OBJ_START);
+			config_set_farm_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
 
-	} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
-			   strcmp(thirdlevel, "") == 0) {
+		} else if (strcmp(firstlevel, CONFIG_KEY_POLICIES) == 0 &&
+				   strcmp(thirdlevel, "") == 0) {
 
-		ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error stopping policy",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
-		}
-		obj_rulerize(OBJ_START_INV);
+			ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "error stopping policy");
+				goto delete_end;
+			}
+			obj_rulerize(OBJ_START_INV);
 
-		ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
-		if (ret < 0) {
-			config_print_response(&state->body_response, "%s%s", "error deleting policy",
-									config_get_output());
-			config_delete_output();
-			state->status_code = WS_HTTP_500;
-			goto delete_end;
+			ret = config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
+			if (ret)
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting policy");
+
+		} else if (strcmp(firstlevel, CONFIG_KEY_ADDRESSES) == 0 &&
+				   strcmp(thirdlevel, "") == 0) {
+
+			ret = config_set_address_action(secondlevel, CONFIG_VALUE_ACTION_STOP);
+			if (ret) {
+				snprintf(message, SRV_MAX_IDENT, "%s", "error stopping address");
+				goto delete_end;
+			}
+			obj_rulerize(OBJ_START_INV);
+
+			ret = config_set_address_action(secondlevel, CONFIG_VALUE_ACTION_DELETE);
+			if (ret)
+				snprintf(message, SRV_MAX_IDENT, "%s", "error deleting address");
+
+		} else {
+			snprintf(message, SRV_MAX_IDENT, "%s", "invalid request");
+			ret = PARSER_STRUCT_FAILED;
 		}
 
 	} else {
-		config_print_response(&state->body_response, "%s%s", "invalid request",
-								config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_404;
-		return 0;
+
+		ret = config_buffer(state->body, ACTION_STOP);
+		switch (ret) {
+		case PARSER_OK:
+			break;
+		case PARSER_STRUCT_FAILED:
+			snprintf(message, SRV_MAX_IDENT, "%s", "the structure is invalid");
+			goto delete_end;
+		case PARSER_OBJ_UNKNOWN:
+			snprintf(message, SRV_MAX_IDENT, "%s", "the object to modify is unknown");
+			goto delete_end;
+		default:
+			snprintf(message, SRV_MAX_IDENT, "%s", "error parsing buffer");
+			goto delete_end;
+		}
+
+		if (obj_rulerize(OBJ_START)) {
+			snprintf(message, SRV_MAX_IDENT, "%s", "error generating rules");
+			ret = PARSER_FAILED;
+		}
 	}
 
-	config_print_response(&state->body_response, "%s%s", "success", config_get_output());
-	config_delete_output();
-	state->status_code = WS_HTTP_200;
-
 delete_end:
+	config_print_response(&state->body_response, "%s%s", message, config_get_output());
+	config_delete_output();
+	state->status_code = parse_to_http_status(ret);
+
 	return 0;
 }
 
 static int send_post_response(struct nftlb_http_state *state)
 {
 	char firstlevel[SRV_MAX_IDENT] = {0};
+	char message[SRV_MAX_IDENT] = {0};
+	int ret = 0;
 
 	sscanf(state->uri, "/%199[^\n]", firstlevel);
 
-	if (init_http_state(state))
-		return -1;
-
 	if ((strcmp(firstlevel, CONFIG_KEY_FARMS) != 0) &&
-		(strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0)) {
-		config_print_response(&state->body_response, "%s%s", "invalid request",
-								config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_404;
-		return 0;
+		(strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0) &&
+		(strcmp(firstlevel, CONFIG_KEY_ADDRESSES) != 0)) {
+		snprintf(message, SRV_MAX_IDENT, "%s", "invalid request");
+		ret = PARSER_OBJ_UNKNOWN;
+		goto post_end;
 	}
 
-	switch (config_buffer(state->body)) {
+	snprintf(message, SRV_MAX_IDENT, "%s", "success");
+	ret = config_buffer(state->body, ACTION_START);
+	switch (ret) {
 	case PARSER_OK:
 		break;
 	case PARSER_STRUCT_FAILED:
-		config_print_response(&state->body_response, "%s%s",
-				      "the structure is invalid", config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_400;
+		snprintf(message, SRV_MAX_IDENT, "%s", "the structure is invalid");
 		goto post_end;
-		break;
 	case PARSER_OBJ_UNKNOWN:
-		config_print_response(&state->body_response, "%s%s",
-				      "the object to modify is unknown", config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_400;
+		snprintf(message, SRV_MAX_IDENT, "%s", "the object to modify is unknown");
 		goto post_end;
-		break;
 	default:
-		config_print_response(&state->body_response, "%s%s",
-				      "error parsing buffer", config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_400;
-		goto post_end;
-		break;
-	}
-
-	if (obj_rulerize(OBJ_START) != 0) {
-		config_print_response(&state->body_response, "%s%s", "error generating rules",
-								config_get_output());
-		config_delete_output();
-		state->status_code = WS_HTTP_500;
+		snprintf(message, SRV_MAX_IDENT, "%s", "error parsing buffer");
 		goto post_end;
 	}
 
-	config_print_response(&state->body_response, "%s%s", "success", config_get_output());
-	config_delete_output();
-	state->status_code = WS_HTTP_200;
+	if (obj_rulerize(OBJ_START)) {
+		snprintf(message, SRV_MAX_IDENT, "%s", "error generating rules");
+		ret = PARSER_FAILED;
+	}
 
 post_end:
+	config_print_response(&state->body_response, "%s%s", message, config_get_output());
+	config_delete_output();
+	state->status_code = parse_to_http_status(ret);
+
+	return 0;
+}
+
+static int send_patch_response(struct nftlb_http_state *state)
+{
+	char firstlevel[SRV_MAX_IDENT] = {0};
+	char secondlevel[SRV_MAX_IDENT] = {0};
+	char thirdlevel[SRV_MAX_IDENT] = {0};
+	char message[SRV_MAX_IDENT] = {0};
+	int ret = 0;
+
+	sscanf(state->uri, "/%199[^/]/%199[^/]/%199[^\n]",
+	       firstlevel, secondlevel, thirdlevel);
+
+	if ((strcmp(firstlevel, CONFIG_KEY_POLICIES) != 0) &&
+		(strcmp(thirdlevel, CONFIG_KEY_ELEMENTS) != 0)) {
+		snprintf(message, SRV_MAX_IDENT, "%s", "invalid request");
+		ret = PARSER_OBJ_UNKNOWN;
+		goto patch_end;
+	}
+
+	if (config_check_policy(secondlevel) != PARSER_OK) {
+		snprintf(message, SRV_MAX_IDENT, "%s", "the object to modify is unknown");
+		ret = PARSER_OBJ_UNKNOWN;
+		goto patch_end;
+	}
+
+	snprintf(message, SRV_MAX_IDENT, "%s", "success");
+	ret = config_buffer(state->body, ACTION_START);
+	switch (ret) {
+	case PARSER_OK:
+		break;
+	case PARSER_STRUCT_FAILED:
+		snprintf(message, SRV_MAX_IDENT, "%s", "the structure is invalid");
+		goto patch_end;
+	case PARSER_OBJ_UNKNOWN:
+		snprintf(message, SRV_MAX_IDENT, "%s", "the object to modify is unknown");
+		goto patch_end;
+	default:
+		snprintf(message, SRV_MAX_IDENT, "%s", "error parsing buffer");
+		goto patch_end;
+	}
+
+	config_set_policy_action(secondlevel, CONFIG_VALUE_ACTION_FLUSH);
+
+	if (obj_rulerize(OBJ_START)) {
+		snprintf(message, SRV_MAX_IDENT, "%s", "error generating rules");
+		ret = PARSER_FAILED;
+	}
+
+patch_end:
+	config_print_response(&state->body_response, "%s%s", message, config_get_output());
+	config_delete_output();
+	state->status_code = parse_to_http_status(ret);
+
 	return 0;
 }
 
@@ -471,6 +561,8 @@ static int send_response(struct nftlb_http_state *state)
 		return send_post_response(state);
 	case WS_DELETE_ACTION:
 		return send_delete_response(state);
+	case WS_PATCH_ACTION:
+		return send_patch_response(state);
 	default:
 		return -1;
 	}
@@ -540,12 +632,11 @@ static void nftlb_read_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	char cli_address[INET6_ADDRSTRLEN + 6]; //max address length + port length
 
 	if (EV_ERROR & revents) {
-		syslog(LOG_ERR, "Server got invalid event from client read");
+		tools_printlog(LOG_ERR, "Server got invalid event from client read");
 		return;
 	}
 	cli = container_of(io, struct nftlb_client, io);
 
-	state.body_response = NULL;
 	create_buf(&buf);
 	size = recv(io->fd, get_buf_data(&buf), DEFAULT_BUFFER_SIZE - 1, 0);
 	if (size < 0)
@@ -554,11 +645,14 @@ static void nftlb_read_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	buf.next = size;
 
 	if (size == 0) {
-		syslog(LOG_DEBUG, "connection closed by client %s\n",
-		       nftlb_client_address(&cli->addr, cli_address));
-		goto end;
+		tools_printlog(LOG_DEBUG, "connection closed by client %s\n",
+					   nftlb_client_address(&cli->addr, cli_address));
+		goto end_no_state;
 	}
 
+	if (init_http_state(&state))
+		goto end_no_state;
+
 	if (get_request(io->fd, &buf, &state) < 0) {
 		nftlb_http_send_response(io, &state, 0);
 		goto end;
@@ -572,18 +666,16 @@ static void nftlb_read_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	nftlb_http_send_response(io, &state, strlen(state.body_response));
 	send(io->fd, state.body_response, strlen(state.body_response), 0);
 
-	syslog(LOG_DEBUG, "connection closed by server %s\n",
-		       nftlb_client_address(&cli->addr, cli_address));
+	tools_printlog(LOG_DEBUG, "connection closed by server %s\n",
+				   nftlb_client_address(&cli->addr, cli_address));
 end:
-	if (state.body_response)
-		free(state.body_response);
+	fin_http_state(&state);
+end_no_state:
 	clean_buf(&buf);
 
 	ev_timer_stop(loop, &cli->timer);
 	nftlb_client_release(loop, cli);
 
-	syslog(LOG_DEBUG, "%d client(s) connected", --nftserver.clients);
-
 	return;
 }
 
@@ -594,8 +686,8 @@ static void nftlb_timer_cb(struct ev_loop *loop, ev_timer *timer, int events)
 
 	cli = container_of(timer, struct nftlb_client, timer);
 
-	syslog(LOG_ERR, "timeout for client %s\n",
-	       nftlb_client_address(&cli->addr, cli_address));
+	tools_printlog(LOG_ERR, "timeout for client %s\n",
+				   nftlb_client_address(&cli->addr, cli_address));
 
 	nftlb_client_release(loop, cli);
 }
@@ -606,22 +698,21 @@ static void accept_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	socklen_t addrlen = sizeof(client_addr);
 	struct nftlb_client *cli;
 	int client_sd;
-	char cli_address[INET6_ADDRSTRLEN + 6]; //max address length + port length
 
 	if (EV_ERROR & revents) {
-		syslog(LOG_ERR, "Server got an invalid event from client");
+		tools_printlog(LOG_ERR, "Server got an invalid event from client");
 		return;
 	}
 
 	client_sd = accept(io->fd, (struct sockaddr *)&client_addr, &addrlen);
 	if (client_sd < 0) {
-		syslog(LOG_ERR, "Server accept error");
+		tools_printlog(LOG_ERR, "Server accept error");
 		return;
 	}
 
 	cli = malloc(sizeof(struct nftlb_client));
 	if (!cli) {
-		syslog(LOG_ERR, "No memory available to allocate new client");
+		tools_printlog(LOG_ERR, "No memory available to allocate new client");
 		return;
 	}
 	memcpy(&cli->addr, &client_addr, sizeof(cli->addr));
@@ -630,10 +721,6 @@ static void accept_cb(struct ev_loop *loop, struct ev_io *io, int revents)
 	ev_io_start(loop, &cli->io);
 	ev_timer_init(&cli->timer, nftlb_timer_cb, NFTLB_CLIENT_TIMEOUT, 0.);
 	ev_timer_start(loop, &cli->timer);
-
-	syslog(LOG_DEBUG, "connection from %s",
-	       nftlb_client_address(&cli->addr, cli_address));
-	syslog(LOG_DEBUG, "%d client(s) connected", ++nftserver.clients);
 }
 
 int server_init(void)
@@ -677,30 +764,26 @@ int server_init(void)
 
 	s = getaddrinfo(host, port, &hints, &result);
 	if (s != 0) {
-		fprintf(stderr, "getaddrinfo: %s\n", gai_strerror(s));
-		syslog(LOG_ERR, "getaddrinfo: %s\n", gai_strerror(s));
+		tools_printlog(LOG_ERR, "getaddrinfo: %s\n", gai_strerror(s));
 		return -1;
 	}
 
 	server_sd = socket(result->ai_family, SOCK_STREAM, 0);
 	if (server_sd < 0) {
-		fprintf(stderr, "Server socket error\n");
-		syslog(LOG_ERR, "Server socket error");
+		tools_printlog(LOG_ERR, "Server socket error");
 		return -1;
 	}
 	setsockopt(server_sd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int));
 
 	if (bind(server_sd, result->ai_addr, result->ai_addrlen) != 0) {
-		fprintf(stderr, "Server bind error\n");
-		syslog(LOG_ERR, "Server bind error");
+		tools_printlog(LOG_ERR, "Server bind error");
 		freeaddrinfo(result);
 		return -1;
 	}
 	freeaddrinfo(result);
 
 	if (listen(server_sd, 2) < 0) {
-		fprintf(stderr, "Server listen error\n");
-		syslog(LOG_ERR, "Server listen error");
+		tools_printlog(LOG_ERR, "Server listen error");
 		return -1;
 	}
 	nftserver.sd = server_sd;
@@ -713,6 +796,7 @@ int server_init(void)
 
 void server_fini(void)
 {
+	events_delete_srv();
 	close(nftserver.sd);
 }
 
@@ -721,7 +805,7 @@ void server_set_host(const char *host)
 	nftserver.host = malloc(strlen(host)+1);
 
 	if (!nftserver.host) {
-		syslog(LOG_ERR, "No memory available to allocate the server host");
+		tools_printlog(LOG_ERR, "No memory available to allocate the server host");
 		return;
 	}
 
@@ -732,7 +816,7 @@ void server_set_port(const char *port)
 {
 	nftserver.port = malloc(strlen(port)+1);
 	if (!nftserver.port) {
-		syslog(LOG_ERR, "No memory available to allocate the server port");
+		tools_printlog(LOG_ERR, "No memory available to allocate the server port");
 		return;
 	}
 
@@ -746,7 +830,7 @@ void server_set_key(char *key)
 	if (!nftserver.key) {
 		nftserver.key = (char *)malloc(SRV_MAX_IDENT);
 		if (!nftserver.key) {
-			syslog(LOG_ERR, "No memory available to allocate the server key");
+			tools_printlog(LOG_ERR, "No memory available to allocate the server key");
 			return;
 		}
 	}
